"""Pin definitions for the FTDI FT2232H"""
from adafruit_blinka.microcontroller.ftdi_mpsse.ft2232h import pin

# MPSSE A
AD4 = pin.AD4
AD5 = pin.AD5
AD6 = pin.AD6
AD7 = pin.AD7
AC0 = pin.AC0
AC1 = pin.AC1
AC2 = pin.AC2
AC3 = pin.AC3
AC4 = pin.AC4
AC5 = pin.AC5
AC6 = pin.AC6
AC7 = pin.AC7

SDA0 = pin.SDA0
SCL0 = pin.SCL0

SCK0 = pin.SCK0
SCLK0 = pin.SCLK0
MOSI0 = pin.MOSI0
MISO0 = pin.MISO0

# MPSSE B
BD4 = pin.BD4
BD5 = pin.BD5
BD6 = pin.BD6
BD7 = pin.BD7
BC0 = pin.BC0
BC1 = pin.BC1
BC2 = pin.BC2
BC3 = pin.BC3
BC4 = pin.BC4
BC5 = pin.BC5
BC6 = pin.BC6
BC7 = pin.BC7

SDA1 = pin.SDA1
SCL1 = pin.SCL1

SCK1 = pin.SCK1
SCLK1 = pin.SCLK1
MOSI1 = pin.MOSI1
MISO1 = pin.MISO1
